/**
 * Functions dealing with DARIAH-DE Storage upload, download, sharing, unsharing, and deleting.
 *
 * TODO: jquery object
 */

/*
 * Create new file, get storage file ID.
 */
function postToDariahStorage(postdata) {
	// Re-set storage ID.
	dsid = "";
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    showMessage('console', '...saving...');
    // Create new file.
    $.ajax({
		url: storageURL,
		type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		contentType: 'text/csv',
		data: postdata,
		success: function(data, status, xhr) {
			//console.log('loc: ' + xhr.getResponseHeader('Location'));
			var location = xhr.getResponseHeader('Location');
			dsid = location.substring(location.lastIndexOf('/') + 1);
            // Just change the URL here, do NO reload of page to stay in corretk workflow!
            // Sorry for the urglimurgli! Really don't know what it's supposed to mean!
            history.replaceState({ urgli: 'murgli' }, 'id',
                window.location.origin + window.location.pathname + '?id=' + dsid);
            newSheetAlert();
            doThingsOnIDExistingCheck();
            setUnsharedStatusOnScreen();
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error creating dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>No new dataset could be created in the DARIAH-DE OwnStorage!</p>';
            newAlert('error', title,  message);
		}
	});
}

/**
 * Load file from DARIAH-DE storage using storage ID.
 */
function loadFromDariahStorage(id) {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    $.ajax({
		url: storageURL + id,
		type: 'GET',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(data) {
            doThingsOnIDExistingCheck();
			csvToTable(data);
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error loading dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The dataset with ID <i>' + id + '</i> could not be loaded from the DARIAH-DE OwnStorage!';
            newAlert('error', title,  message);
            // Re-set storage ID.
        	dsid = "";
            doThingsOnIDExistingCheck();
            showFirstStartWizard();
		}
	});
}

/**
 * Load file from DARIAH-DE storage using storage ID, check shared status.
 */
function checkSharedStatus(id) {
    // Assemble logID.
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    $.ajax({
		url: storageURL + id,
		type: 'GET',
        headers: { 'X-Transaction-ID': logID },
		success: function(data) {
            // Public datasets already are shared!
            setSharedStatusOnScreen();
		},
        error: function(xhr, textStatus, errorThrown) {
            // Nonpublic datasets are not yet shared, so we are allowed to share them.
            setUnsharedStatusOnScreen();
        }
	});
}

/*
 * Update a dataset (file) using ID and post data.
 */
function updateDariahStorage(id, postdata) {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    showMessage('console', '...saving...');
	// Update file.
    $.ajax({
		url: storageURL + id,
		type: 'PUT',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		contentType: 'text/csv',
		data: postdata,
        success: function(response) {
            showMessage('console', 'All changes saved.');
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error updating dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The Dataset with ID <strong>' + id + '</strong> could not be updated to the DARIAH-DE OwnStorage!</p>';
            newAlert('error', title,  message);
		}
	});
}

/**
 * Share a dataset using the publish() method of the DARIAH-DE PDP using DARIAH-DE OwnStorage.
 */
function publishDataset() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Share file.
    $.ajax({
		url: storageURL + dsid + "/publish",
        type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            var title = 'Dataset successfully shared!';
            var accessURL =  storageURL + dsid;
            var message = '<p>The Dataset with ID <strong>' + dsid + '</strong> has been shared successfully!<br/> It can now be accessed by everyone using the URL <a href="' + accessURL + '">' + accessURL + '</a> by tools such as the DARIAH-DE Geo-Browser or simply for downloading via browser.</p>';
            newAlert('success', title,  message);
            setSharedStatusOnScreen();
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error sharing dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The Dataset with ID <strong>' + dsid + '</strong> could not be shared! Maybe it has to be unshared first!</p>';
            newAlert('error', title,  message);
		}
	});
}

/**
 * Unshare a dataset (again) using the unpublish() method of the DARIAH-DE PDP using DARIAH-DE OwnStorage.
 */
function unpublishDataset() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Share file.
    $.ajax({
		url: storageURL + dsid + "/unpublish",
        type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            var title = 'Dataset successfully unshared!';
            var message = '<p>The dataset with ID <strong>' + dsid + '</strong> has been unshared successfully!<br/> It can now only accessed by you again!</p>';
            newAlert('success', title,  message);
            setUnsharedStatusOnScreen();
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error unsharing dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The dataset with ID <strong>' + dsid + '</strong> could not be unshared!</p>';
            newAlert('error', title,  message);
		}
	});
}

/**
 * Unpublish a dataset first, then delete.
 */
function unpublishAndDeleteDataset() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Share file.
    $.ajax({
		url: storageURL + dsid + "/unpublish",
        type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            deleteDatasetFromStorage();
		},
        error: function(xhr, textStatus, errorThrown) {
            deleteDatasetFromStorage();
		}
	});
}

/**
 * Delete a dataset from the DARIAH-DE OwnStorage.
 */
function deleteDatasetFromStorage() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Delete file.
	$.ajax({
		url: storageURL + dsid,
		type: 'DELETE',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            var title = 'Dataset successfully deleted!';
            var message = '<p>The dataset with ID <strong>' + dsid + '</strong> has been deleted successfully!</p>';
            newAlert('success', title,  message);
            // Re-set storage ID.
        	dsid = "";
            doThingsOnIDExistingCheck();
            showFirstStartWizard();
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error deleting dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The dataset with ID <strong>' + dsid + '</strong> could not be deleted!</p>';
            checkSharedStatus(dsid);
            newAlert('error', title,  message);
		}
	});
}

/**
 * Check if token is existing, if not get one!
 */
function authenticate(action) {
    // Present redirection message.
    var title = 'Redirecting to DARIAH AAI';
    var message = '<p>You are being redirected to the DARIAH AAI to get you a DARIAH-DE StorageToken to '
        + action + ' a new dataset</p>';
    newAlert('success', title,  message);
    // Redirect to PDP if no token is existing in sessionStorage.
    window.location = pdpURL + window.location.href + '?action=' + action;
}
